using System;
using ODE.Integrators;

namespace ODESolverTest
{
	class MainClass
	{
		public static void TestIntegrator(IIntegrator integrator)
		{
			Console.WriteLine("Testing integrator '{0}'", integrator);
			
			double x = 0; // time (s)
			double[] y = { 0, 13 }; // position, velocity (ms)
			double h = 0.1; // timestep
			
			integrator.SetInitialState(x, y, h);
			
			bool stopped = false;
			while (!stopped) {
				integrator.Update(
					delegate(double _x, double[] _y, out double[] dydx)
					{
						dydx = new double[2];
						dydx[0] = _y[1]; // dx/dt = v
						dydx[1] = -9.81; // dv/dt = a in ms^-2
					},
					delegate(double _x, double[] _y, out StoppingCriteriaDirection direction)
					{
						direction = StoppingCriteriaDirection.Negative;
						return _y[0];
					}, out stopped);
				integrator.GetState(out x, out y);
			}
			
			Console.WriteLine("Time: {0:F6}\tPosition: {1:F6}\tVelocity: {2:F6}", x, y[0], y[1]);
			Console.WriteLine();
		}

		public static void TestIntegrator2(IIntegrator integrator)
		{
			Console.WriteLine("Testing integrator '{0}'", integrator);
			
			double x = 0; // time (s)
			double[] y = { 0 }; // position, velocity (ms)
			double h = 0.1; // timestep
			
			integrator.SetInitialState(x, y, h);
			
			bool stopped = false;
			int derivCount = 0;
			while (!stopped) {
				integrator.Update(
					delegate(double _x, double[] _y, out double[] dydx)
					{
						derivCount++;
						dydx = new double[1];
						dydx[0] = 1 + _y[0]*_y[0]; // dx/dt = 1 + x^2
					},
					delegate(double _x, double[] _y, out StoppingCriteriaDirection direction)
					{
						direction = StoppingCriteriaDirection.Negative;
						return 1.4 - _x;
					}, out stopped);
				integrator.GetState(out x, out y);
			}
			
			Console.WriteLine("Time: {0:F6}\ty: {1:F6}\tref: {2:F6}\terror: {3}", x, y[0], Math.Tan(x), y[0] - Math.Tan(x));
			Console.WriteLine();
		}
		
		public static void Main(string[] args)
		{
			Console.WriteLine();
			Console.WriteLine("--------------------- First test ----------------------");
			Console.WriteLine();

			TestIntegrator(new EulerIntegrator());
			TestIntegrator(new RK2Integrator());
			TestIntegrator(new RK4Integrator());

			Console.WriteLine();
			Console.WriteLine("--------------------- Second test ---------------------");
			Console.WriteLine();
			
			TestIntegrator2(new EulerIntegrator());
			TestIntegrator2(new RK2Integrator());
			TestIntegrator2(new RK4Integrator());
		}
	}
}