
using System;

namespace ODE.Integrators
{
	
	
	/// <summary>
	/// 4th order Runge-Kutta integrator
	/// </summary>
	public class RK4Integrator : IntegratorBase
	{
		
		public RK4Integrator()
		{
		}
		
		public override void Update(DerivativesDelegate derivatives) 
		{
			
			double[] k1, k2, k3, k4;
			
			double[] yt = new double[n];
			
			double h2 = h / 2;
			double h6 = h / 6;
			double x2 = x + h2;
			double xh = x + h;
			
			// compute intermediary steps
			// get derivatives at x
			derivatives(x, y, out k1);
			
			// get derivatives at x+h/2
			for (int i = 0; i < n; i++) {
				yt[i] = y[i] + h2*k1[i];
			}
			derivatives(x2, yt, out k2);
			
			// find new derivatives at x+h/2
			for (int i = 0; i < n; i++) {
				yt[i] = y[i] + h2*k2[i];
			}
			derivatives(x2, yt, out k3);
			
			// find derivatives at x+h
			for (int i = 0; i < n; i++) {
				yt[i] = y[i] + h*k3[i];
			}
			derivatives(xh, yt, out k4);
			
			// get final state by using weighted 
			// average of derivatives
			for (int i = 0; i < n; i++) {
				y[i] = y[i] + h6*(k1[i] + 2*k2[i] + 2*k3[i] + k4[i]);
			}
			
			x = xh;
		}
	}
}
